local deformat = AceLibrary("Deformat-2.0")
local tablet = AceLibrary("Tablet-2.0")
local roster = AceLibrary("Roster-2.1")
local BC = AceLibrary("Babble-Class-2.2")
local BB = AceLibrary("Babble-Boss-2.2")
local _G = getfenv(0)

-- Functions
local setFrameHeader = nil
local setText = nil

local currentTrackset = nil
local trackerFrame = nil
local grouped = nil
local lastBossKilled = nil

local coloredName = setmetatable({}, {__index =
	function(self, key)
		if type(key) == "nil" then return nil end
		local class = roster.roster[key] and roster.roster[key].class or "WTF"
		self[key] = "|cff" .. BC:GetHexColor(class) .. key .. "|r"
		return self[key]
	end
})

local L = AceLibrary("AceLocale-2.2"):new("NRT")
L:RegisterTranslations("enUS", function() return {
	["%s killed. Take raid attendance now?"] = true,
	["NRT: Whisper me your mains name to be added to the DKP list."] = true,
	["NRT: %s down! Whisper me your mains name to be added to the DKP list."] = true,
	["NRT: If you haven't whispered me by now, you're too late."] = true,
	["Player"] = true,
	["Date"] = true,
	["Item"] = true,
	["From"] = true,
	["Tracked"] = true,
	["Take raid attendance now"] = true,
	["|cffeda55fClick|r an item to show. |cffeda55fShift-Click|r to remove."] = true,
	["%s has been added to the attendance list."] = true,
	["%s is already in the raid."] = true,
	["%s is not in the guild."] = true,
	["%s is already in the attendance list."] = true,
	["Add player"] = true,
	["Adds a player to the latest attendance set."] = true,
	["%s looted %s (%s)!"] = true,
} end)

NRT = AceLibrary("AceAddon-2.0"):new(
	"AceEvent-2.0",
	"AceConsole-2.0",
	"AceDB-2.0",
	"FuBarPlugin-2.0",
	"AceHook-2.1"
)
local addon = NRT

local options = {
	type = "group",
	args = {
		add = {
			type = "text",
			name = L["Add player"],
			desc = L["Adds a player to the latest attendance set."],
			get = false,
			usage = "<player name>",
			set = function(newPlayer)
				newPlayer = newPlayer:trim():lower()
				local size = #addon.db.profile.tracked
				local set = addon.db.profile.tracked[size]
				if set then
					local found = nil
					for i, v in ipairs(set.attendants) do
						if v:lower() == newPlayer then
							found = true
							break
						end
					end
					if not found then
						table.insert(set.attendants, newPlayer)
					end
				end
			end,
			disabled = function() return type(addon.db.profile.tracked) ~= "table" or #addon.db.profile.tracked == 0 end,
		},
	},
}

addon.hasIcon = "Interface\\Icons\\Spell_Holy_DivineIntervention"

function addon:OnInitialize()
	local revision = tonumber((string.match("$Revision: 30466 $", "^.-(%d+).-$"))) or 1
	if not self.version then self.version = "1.0" end
	self.version = self.version .. "." .. revision
	self.revision = revision

	self.clickableTooltip = true
	self.hasNoColor = true

	_G.NRT_VERSION = self.version
	_G.NRT_REVISION = self.revision

	self:RegisterDB("NRTDB")
	self:RegisterDefaults("profile", {
		loot = {},
		tracked = {},
	})

	self.OnMenuRequest = options
end

function addon:OnEnable(first)
	if first then
		local popup = _G.StaticPopupDialogs
		if type(popup) ~= "table" then
			popup = {}
		end
		if type(popup["NRTBossKilled"]) ~= "table" then
			popup["NRTBossKilled"] = {
				text = "",
				button1 = TEXT(YES),
				button2 = TEXT(NO),
				timeout = 0,
				whileDead = 1,
				hideOnEscape = 1,
				OnAccept = function() self:TakeAttendance() end
			}
		end
	end

	self:RegisterEvent("RosterLib_RosterUpdated")
	self:CreateTrackerFrame()
end

function addon:CreateTrackerFrame()
	if trackerFrame then return end
	trackerFrame = CreateFrame("Frame", "NRTTrackerFrame", UIParent)
	trackerFrame:Hide()

	trackerFrame:SetWidth(280)
	trackerFrame:SetHeight(420)
	trackerFrame:SetBackdrop({
		bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 16,
		edgeFile = "Interface\\AddOns\\NRT\\textures\\otravi-semi-full-border", edgeSize = 32,
		insets = {left = 1, right = 1, top = 20, bottom = 1},
	})

	trackerFrame:SetBackdropColor(24/255, 24/255, 24/255)
	trackerFrame:ClearAllPoints()
	trackerFrame:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
	trackerFrame:EnableMouse(true)
	trackerFrame:RegisterForDrag("LeftButton")
	trackerFrame:SetMovable(true)
	trackerFrame:SetScript("OnDragStart", function()
		this:StartMoving()
	end)
	trackerFrame:SetScript("OnDragStop", function()
		this:StopMovingOrSizing()
	end)

	local cheader = trackerFrame:CreateFontString(nil,"OVERLAY")
	cheader:ClearAllPoints()
	cheader:SetWidth(240)
	cheader:SetHeight(15)
	cheader:SetPoint("TOP", trackerFrame, "TOP", 0, -14)
	cheader:SetFont("Fonts\\FRIZQT__.TTF", 12)
	cheader:SetJustifyH("LEFT")
	setFrameHeader = function(text)
		cheader:SetText(text)
	end
	cheader:SetText("Tracker")
	cheader:SetShadowOffset(.8, -.8)
	cheader:SetShadowColor(0, 0, 0, 1)

	local close = trackerFrame:CreateTexture(nil, "ARTWORK")
	close:SetTexture("Interface\\AddOns\\NRT\\textures\\otravi-close")
	close:SetTexCoord(0, .625, 0, .9333)
	close:SetWidth(20)
	close:SetHeight(14)
	close:SetPoint("TOPRIGHT", trackerFrame, "TOPRIGHT", -7, -15)

	local closebutton = CreateFrame("Button", nil)
	closebutton:SetParent( trackerFrame )
	closebutton:SetWidth(20)
	closebutton:SetHeight(14)
	closebutton:SetPoint("CENTER", close, "CENTER")
	closebutton:SetScript( "OnClick", function() trackerFrame:Hide() end )

	local scroll = CreateFrame("ScrollFrame", "NRTScrollFrame", trackerFrame, "UIPanelScrollFrameTemplate")
	scroll:SetPoint("TOPLEFT", trackerFrame, "TOPLEFT", 20, -40)
	scroll:SetPoint("RIGHT", trackerFrame, "RIGHT", -40, 0)
	scroll:SetPoint("BOTTOM", trackerFrame, 0, 20)

	local textField = CreateFrame("EditBox", "NRTEditBox", scroll)
	textField:SetFontObject(ChatFontNormal)
	textField:SetMultiLine(true)
	textField:SetWidth(240)
	textField:SetScript("OnEscapePressed", function() this:ClearFocus() end)
	textField:SetScript("OnTextChanged", function() scroll:UpdateScrollChildRect() end)
	textField:SetText("")
	setText = function(text)
		textField:SetText(text)
	end
	textField:SetAutoFocus(false)
	scroll:SetScrollChild(textField)
end

function addon:JoinedRaid()
	if grouped then return end
	grouped = true

	self:RegisterEvent("CHAT_MSG_COMBAT_HOSTILE_DEATH")
	self:RegisterEvent("CHAT_MSG_LOOT")
end

function addon:LeftRaid()
	if not grouped then return end
	grouped = nil

	self:UnregisterEvent("CHAT_MSG_COMBAT_HOSTILE_DEATH")
	self:UnregisterEvent("CHAT_MSG_LOOT")
end

function addon:TakeAttendance()
	local name = nil
	if lastBossKilled then
		name = lastBossKilled
	else
		name = GetRealZoneText()
	end
	local newTrackset = {
		name = name,
		zone = GetRealZoneText(),
		attendants = {},
		date = date("%d/%m %H:%M"),
	}
	for n, u in pairs(roster.roster) do
		if u.class ~= "PET" then
			table.insert(newTrackset.attendants, n)
		end
	end
	table.insert(self.db.profile.tracked, newTrackset)
	currentTrackset = #self.db.profile.tracked

	self:Hook("ChatFrame_MessageEventHandler", "WhisperHandler", true)
	local str = nil
	if lastBossKilled then
		str = L["NRT: %s down! Whisper me your mains name to be added to the DKP list."]:format(lastBossKilled)
	else
		str = L["NRT: Whisper me your mains name to be added to the DKP list."]
	end
	SendChatMessage(("*"):rep(10), "GUILD")
	SendChatMessage(str, "GUILD")
	SendChatMessage(("*"):rep(10), "GUILD")

	local id, name = GetChannelName("nstandby")
	if id then
		SendChatMessage(("*"):rep(10), "CHANNEL", nil, id)
		SendChatMessage(str, "CHANNEL", nil, id)
		SendChatMessage(("*"):rep(10), "CHANNEL", nil, id)
	end
	self:ScheduleEvent(self.AttendanceTimeout, 120, self)

	if type(ShutUp_Disable) == "function" then
		ShutUp_Disable()
	end
end

function addon:AttendanceTimeout()
	self:Unhook("ChatFrame_MessageEventHandler")
	local str = L["NRT: If you haven't whispered me by now, you're too late."]
	SendChatMessage(str, "GUILD")
	local id, name = GetChannelName("nstandby")
	if id then
		SendChatMessage(str, "CHANNEL", nil, id)
	end
	currentTrackset = nil
	if type(ShutUp_Enable) == "function" then
		ShutUp_Enable()
	end
end

function addon:RosterLib_RosterUpdated()
	if not grouped and GetNumRaidMembers() > 0 then
		self:JoinedRaid()
	elseif grouped and GetNumRaidMembers() == 0 then
		self:LeftRaid()
	end
end

function addon:CHAT_MSG_LOOT(msg)
	local player, item = deformat(msg, LOOT_ITEM)
	if not player then
		item = deformat(msg, LOOT_ITEM_SELF)
		if item then
			player = UnitName("player")
		end
	end

	if type(item) == "string" and type(player) == "string" then
		local itemName, itemLink, itemRarity = GetItemInfo(item)
		if itemRarity > 3 then -- Epic or higher
			-- Find the item ID to make sure it's not a void crystal (22450)
			local itemId = select(3, itemLink:find("item:(%d+):"))
			if not itemId then return end
			itemId = tonumber(itemId:trim())
			if type(itemId) ~= "number" or itemId == 22450 then return end

			local unitId = roster:GetUnitObjectFromName(player).unitid

			local lootedFrom = nil
			if UnitExists(unitId .. "target") and (UnitClassification(unitId .. "target") == "worldboss" or UnitClassification(unitId .. "target") == "elite") then
				lootedFrom = UnitName(unitId .. "target")
			elseif lastBossKilled then
				lootedFrom = lastBossKilled
			else
				lootedFrom = GetRealZoneText()
			end

			SendChatMessage(L["%s looted %s (%s)!"]:format(player, itemLink, lootedFrom), "GUILD")

			table.insert(self.db.profile.loot, {
				player = player,
				item = itemLink,
				from = lootedFrom,
				date = date("%d/%m %H:%M"),
			})
		end
	end
end

function addon:CHAT_MSG_COMBAT_HOSTILE_DEATH(msg)
	local mob = deformat(msg, UNITDIESOTHER)
	if mob and BB:HasReverseTranslation(mob) then
		lastBossKilled = mob
		_G.StaticPopupDialogs["NRTBossKilled"].text = L["%s killed. Take raid attendance now?"]:format(mob)
		_G.StaticPopup_Show("NRTBossKilled")
		self:ScheduleEvent(function()
			lastBossKilled = nil
		end, 600)
	end
end

local guildMemberList = nil
local function IsGuildMember(name)
	if type(name) ~= "string" then return end
	if not guildMemberList then
		guildMemberList = {}

		local offline = GetGuildRosterShowOffline()
		local selection = GetGuildRosterSelection()
		SetGuildRosterShowOffline(true)
		SetGuildRosterSelection(0)
		GetGuildRosterInfo(0)
		local numGuildMembers = GetNumGuildMembers()
		for i = 1, numGuildMembers, 1 do
			local name = GetGuildRosterInfo(i)
			if name then
				guildMemberList[name:lower()] = true
			end
		end
		SetGuildRosterShowOffline(offline)
		SetGuildRosterSelection(selection)
	end
	return guildMemberList[name:lower()]
end

local function IsGroupMember(name)
	name = name:lower()
	local found = nil
	for n, u in pairs(roster.roster) do
		if n:lower() == name then
			found = true
			break
		end
	end
	return found
end

function addon:WhisperHandler(event, ...)
	if event == "CHAT_MSG_WHISPER" and type(arg1) == "string" and type(arg2) == "string" then
		local player = arg2
		local text = arg1
		if not text:find("%s") then
			text = text:lower()
			if IsGroupMember(text) then
				SendChatMessage(L["%s is already in the raid."]:format(text), "WHISPER", nil, player)
			elseif IsGuildMember(text) then
				local set = self.db.profile.tracked[currentTrackset]
				if set then
					local found = nil
					for i, v in ipairs(set.attendants) do
						if v:lower() == text then
							found = true
							break
						end
					end
					if not found then
						table.insert(set.attendants, text)
						SendChatMessage(L["%s has been added to the attendance list."]:format(text), "WHISPER", nil, player)
					end
				end
			else
				SendChatMessage(L["%s is not in the guild."]:format(text), "WHISPER", nil, player)
			end
		end
	end
	return self.hooks["ChatFrame_MessageEventHandler"](event, ...)
end

local function itemClickFunc(item, index)
	if IsShiftKeyDown() and type(index) == "number" then
		table.remove(addon.db.profile.loot, index)
	elseif item then
		SetItemRef(item)
	end
end

function addon:ShowTrackerForRaidIndex(index)
	local raid = self.db.profile.tracked[index]
	if not raid then return end
	local text = table.concat(raid.attendants, "\n")
	setText(text)
	setFrameHeader(raid.date .. " " .. raid.name)
	trackerFrame:Show()
end

local function trackerClickFunc(trackId)
	if IsShiftKeyDown() and type(trackId) == "number" then
		if addon.db.profile.tracked[trackId] then
			table.remove(addon.db.profile.tracked, trackId)
		end
	elseif type(trackId) == "number" then
		-- Show the text area with the details from this track
		addon:ShowTrackerForRaidIndex(trackId)
	end
end

function addon:OnTooltipUpdate()
	local cat = tablet:AddCategory(
		"columns", 2,
		"text", L["Date"],
		"text2", L["Tracked"],
		"child_textR", 1,
		"child_textG", 1,
		"child_textB", 0,
		"child_text2R", 1,
		"child_text2G", 1,
		"child_text2B", 1,
		"showWithoutChildren", false,
		"child_func", trackerClickFunc
	)
	for i, v in ipairs(self.db.profile.tracked) do
		cat:AddLine(
			"text", v.date,
			"text2", v.name,
			"arg1", i
		)
	end
	cat = tablet:AddCategory()
	cat:AddLine(
		"text", L["Take raid attendance now"],
		"func", function() self:TakeAttendance() end
	)
	cat = tablet:AddCategory(
		"columns", 4,
		"text", L["Date"],
		"text2", L["Player"],
		"text3", L["Item"],
		"text4", L["From"],
		"child_textR", 1,
		"child_textG", 1,
		"child_textB", 0,
		"child_text2R", 1,
		"child_text2G", 1,
		"child_text2B", 1,
		"showWithoutChildren", false,
		"child_func", itemClickFunc
	)
	for i, v in ipairs(self.db.profile.loot) do
		cat:AddLine(
			"text", v.date,
			"text2", coloredName[v.player],
			"text3", v.item,
			"text4", v.from,
			"arg1", v.item,
			"arg2", i
		)
	end

	tablet:SetHint(L["|cffeda55fClick|r an item to show. |cffeda55fShift-Click|r to remove."])
end

